# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestUpdateAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        self.application.add_page(self.new_page())

    def test_flash_message(self):
        self.application.flash_message("")
        self.application.flash_message(None)

    def test__on_activate_next_project_activate(self):
        self.application.notebook.set_current_page(0)
        self.application.get_action("activate-next-project").activate()

    def test__on_activate_previous_project_activate(self):
        self.application.notebook.set_current_page(-1)
        self.application.get_action("activate-previous-project").activate()

    def test__on_move_tab_left_activate(self):
        self.application.notebook.set_current_page(-1)
        self.application.get_action("move-tab-left").activate()

    def test__on_move_tab_right_activate(self):
        self.application.notebook.set_current_page(0)
        self.application.get_action("move-tab-right").activate()

    def test_show_message(self):
        self.application.show_message("")
        self.application.show_message(None)

    def test_update_gui(self):
        self.application.update_gui()
        self.application.close_all()
        self.application.update_gui()



# -*- coding: utf-8 -*-

# Copyright (C) 2006 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import gaupol

from gi.repository import Gtk


class TestModule(gaupol.TestCase):

    def test_connect_text_view(self):
        text_view = Gtk.TextView()
        gaupol.ruler.connect_text_view(text_view)
        text_buffer = text_view.get_buffer()
        text_buffer.insert_at_cursor("test\ntest")

    def test_disconnect_text_view(self):
        text_view = Gtk.TextView()
        gaupol.ruler.connect_text_view(text_view)
        gaupol.ruler.disconnect_text_view(text_view)
        text_buffer = text_view.get_buffer()
        text_buffer.insert_at_cursor("test\ntest")

    def test_get_length_function__char(self):
        unit = gaupol.length_units.CHAR
        function = gaupol.ruler.get_length_function(unit)
        assert function("iii") == 3

    def test_get_length_function__em(self):
        unit = gaupol.length_units.EM
        function = gaupol.ruler.get_length_function(unit)
        assert 0.5 < function("M") < 1.5

    def test_get_lengths__char(self):
        unit = gaupol.length_units.CHAR
        gaupol.conf.editor.length_unit = unit
        lengths = gaupol.ruler.get_lengths("MMM\n<i>iii</i>")
        assert tuple(lengths) == (3, 3)

    def test_get_lengths__em(self):
        unit = gaupol.length_units.EM
        gaupol.conf.editor.length_unit = unit
        lengths = gaupol.ruler.get_lengths("MMM\n<i>iii</i>")
        assert -1 < lengths[1] < 1 < lengths[0]



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import gaupol

from gi.repository import Gtk


class TestMultilineCellRenderer(gaupol.TestCase):

    def run_renderer(self):
        tree_view = Gtk.TreeView()
        tree_view.set_headers_visible(False)
        store = Gtk.ListStore(str)
        store.append(("test\ntest test",))
        tree_view.set_model(store)
        self.renderer.props.editable = True
        column = Gtk.TreeViewColumn("", self.renderer, text=0)
        tree_view.append_column(column)
        window = Gtk.Window()
        window.connect("delete-event", Gtk.main_quit)
        window.set_position(Gtk.WindowPosition.CENTER)
        window.set_default_size(240, 70)
        window.add(tree_view)
        window.show_all()
        Gtk.main()

    def setup_method(self, method):
        self.renderer = gaupol.MultilineCellRenderer()


class TestMultilineDiffCellRenderer(gaupol.TestCase):

    def run_renderer(self):
        tree_view = Gtk.TreeView()
        tree_view.set_headers_visible(False)
        store = Gtk.ListStore(str, str)
        store.append(("test", "test\ntest"))
        tree_view.set_model(store)
        self.renderer.props.editable = True
        column = Gtk.TreeViewColumn("", self.renderer, text=1, ref_text=0)
        tree_view.append_column(column)
        window = Gtk.Window()
        window.connect("delete-event", Gtk.main_quit)
        window.set_position(Gtk.WindowPosition.CENTER)
        window.set_default_size(240, 70)
        window.add(tree_view)
        window.show_all()
        Gtk.main()

    def setup_method(self, method):
        self.renderer = gaupol.MultilineDiffCellRenderer()



# -*- coding: utf-8 -*-

# Copyright (C) 2017 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon

from aeidon.markups.test.test_subrip import TestSubRip


class TestWebVTT(TestSubRip):

    text = ("All things weird are normal\n"
            "in this whore of cities.")

    def setup_method(self, method):
        self.markup = aeidon.markups.new(aeidon.formats.WEBVTT)

    def test_colorize(self):
        pass

    def test_decode__color(self):
        pass

    def test_encode__color(self):
        text = ("<color=#ccccff>All things weird are normal\n"
                "in this whore of cities.</color>")
        assert self.markup.encode(text) == self.text



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon

MAIN = aeidon.documents.MAIN
TRAN = aeidon.documents.TRAN


class TestRegisterAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()
        self.delegate = self.project.undo.__self__

    def test_redo(self):
        text_0 = self.project.subtitles[0].main_text
        text_1 = self.project.subtitles[1].main_text
        text_2 = self.project.subtitles[2].main_text
        self.project.clear_texts((0,), MAIN)
        self.project.clear_texts((1,), MAIN)
        self.project.clear_texts((2,), MAIN)
        self.project.undo(3)
        assert self.project.subtitles[0].main_text == text_0
        assert self.project.subtitles[1].main_text == text_1
        assert self.project.subtitles[2].main_text == text_2
        self.project.redo(1)
        self.project.redo(2)
        assert self.project.subtitles[0].main_text == ""
        assert self.project.subtitles[1].main_text == ""
        assert self.project.subtitles[2].main_text == ""

    def test_undo(self):
        text_0 = self.project.subtitles[0].main_text
        text_1 = self.project.subtitles[1].main_text
        text_2 = self.project.subtitles[2].main_text
        self.project.clear_texts((0,), MAIN)
        self.project.clear_texts((1,), MAIN)
        self.project.clear_texts((2,), MAIN)
        self.project.undo(1)
        self.project.undo(2)
        assert self.project.subtitles[0].main_text == text_0
        assert self.project.subtitles[1].main_text == text_1
        assert self.project.subtitles[2].main_text == text_2
        self.project.redo(3)
        assert self.project.subtitles[0].main_text == ""
        assert self.project.subtitles[1].main_text == ""
        assert self.project.subtitles[2].main_text == ""



# -*- coding: utf-8 -*-

# Copyright (C) 2019 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol

from gi.repository import Gtk


class TestSpellChecker(aeidon.TestCase):

    def run_checker(self):
        text_view = Gtk.TextView()
        text_buffer = text_view.get_buffer()
        text_buffer.set_text("It's a humanoid weapon-system\ncreated by the URM Technarchy.")
        self.checker.attach(text_view)
        window = Gtk.Window()
        window.connect("delete-event", Gtk.main_quit)
        window.set_position(Gtk.WindowPosition.CENTER)
        window.set_default_size(400, 200)
        window.add(text_view)
        window.show_all()
        Gtk.main()

    def setup_method(self, method):
        language = self.get_spell_check_language("en")
        self.checker = gaupol.SpellChecker(language)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestFormatAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()
        page = self.application.get_current_page()
        page.view.set_focus(0, page.view.columns.MAIN_TEXT)
        page.view.select_rows((0, 1, 2))

    def test__on_toggle_dialogue_dashes_activate(self):
        self.application.get_action("toggle-dialogue-dashes").activate()

    def test__on_toggle_italicization_activate(self):
        self.application.get_action("toggle-italicization").activate()

    def test__on_use_lower_case_activate(self):
        self.application.get_action("use-lower-case").activate()

    def test__on_use_sentence_case_activate(self):
        self.application.get_action("use-sentence-case").activate()

    def test__on_use_title_case_activate(self):
        self.application.get_action("use-title-case").activate()

    def test__on_use_upper_case_activate(self):
        self.application.get_action("use-upper-case").activate()



# -*- coding: utf-8 -*-

# Copyright (C) 2006 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class PuppetMaster:

    def __init__(self):
        self.name = "master"


class TestDelegate(aeidon.TestCase):

    def setup_method(self, method):
        self.master = PuppetMaster()
        self.delegate = aeidon.Delegate(self.master)

    def test___getattr__(self):
        assert self.delegate.name == "master"

    def test___setattr____delegate(self):
        self.delegate.none = None
        assert "none" in self.delegate.__dict__
        assert not "none" in self.master.__dict__

    def test___setattr____master(self):
        self.delegate.name = "slave"
        assert self.master.name == "slave"



# -*- coding: utf-8 -*-

# Copyright (C) 2019 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestSpellChecker(aeidon.TestCase):

    def setup_method(self, method):
        language = self.get_spell_check_language("en")
        self.checker = aeidon.SpellChecker(language)

    def test_add_replacement(self):
        self.checker.add_replacement("abc", "xyz")
        suggestions = self.checker.suggest("abc")
        assert suggestions and suggestions[0] == "xyz"

    def test_add_to_session(self):
        assert not self.checker.check("asdf")
        self.checker.add_to_session("asdf")
        assert self.checker.check("asdf")

    def test_available(self):
        assert aeidon.SpellChecker.available()

    def test_check(self):
        assert self.checker.check("subtitle")
        assert not self.checker.check("substitle")

    def test_list_languages(self):
        assert aeidon.SpellChecker.list_languages()

    def test_suggest(self):
        assert self.checker.suggest("substitle")


class TestSpellCheckNavigator(aeidon.TestCase):

    def setup_method(self, method):
        language = self.get_spell_check_language("en")
        self.navigator = aeidon.SpellCheckNavigator(language)
        self.navigator.text = "She knows the fighting\ntechniques of Panzr Kunst."

    def test_ignore(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        self.navigator.ignore()
        error = next(self.navigator)
        assert error == (43, "Kunst")
        self.navigator.ignore()
        self.assert_raises(StopIteration, self.navigator.__next__)

    def test_ignore_all(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        self.navigator.ignore_all()
        self.navigator.reset(self.navigator.text)
        error = next(self.navigator)
        assert error == (43, "Kunst")

    def test_join_with_next(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        self.navigator.join_with_next()
        error = next(self.navigator)
        assert error == (37, "PanzrKunst")

    def test_join_with_previous(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        self.navigator.ignore()
        error = next(self.navigator)
        assert error == (43, "Kunst")
        self.navigator.join_with_previous()
        error = next(self.navigator)
        assert error == (37, "PanzrKunst")

    def test_leading_context(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        assert self.navigator.leading_context(3) == "of "

    def test_replace(self):
        text = self.navigator.text
        text = text.replace("Panzr", "Test")
        error = next(self.navigator)
        assert error == (37, "Panzr")
        self.navigator.replace("Test")
        assert self.navigator.text == text

    def test_replace_all(self):
        self.navigator.reset("abc abc abc")
        error = next(self.navigator)
        self.navigator.replace_all("xyz")
        for error in self.navigator: pass
        assert self.navigator.text == "xyz xyz xyz"

    def test_reset(self):
        error = next(self.navigator)
        self.navigator.reset(self.navigator.text)
        assert next(self.navigator) == error

    def test_suggest(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        assert self.navigator.suggest()

    def test_trailing_context(self):
        error = next(self.navigator)
        assert error == (37, "Panzr")
        assert self.navigator.trailing_context(3) == " Ku"


class TestSpellCheckTokenizer(aeidon.TestCase):

    def setup_method(self, method):
        text = "- This is an URM ship?\n- Uh-huh."
        self.tokenizer = aeidon.SpellCheckTokenizer(text)

    def test_tokenize(self):
        assert list(self.tokenizer.tokenize()) == [
            ( 2, "This"),
            ( 7, "is"),
            (10, "an"),
            (13, "URM"),
            (17, "ship"),
            (25, "Uh"),
            (28, "huh"),
        ]



# -*- coding: utf-8 -*-

# Copyright (C) 2011 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import re


class TestLiner(aeidon.TestCase):

    def setup_method(self, method):
        self.liner = aeidon.Liner(re.compile(r"<.+?>"))
        self.liner.set_penalties((
            dict(pattern=r"( )- ",
                 flags=re.DOTALL|re.MULTILINE,
                 group=1,
                 value=-1000),

            dict(pattern=r"[,.;:!?]( )",
                 flags=re.DOTALL|re.MULTILINE,
                 group=1,
                 value=-100),

            dict(pattern=r"\b(by|the|into|a)( )",
                 flags=re.DOTALL|re.MULTILINE,
                 group=2,
                 value=1000)))

    def test_break_lines__1(self):
        text = "Hello."
        self.liner.set_text(text)
        assert self.liner.break_lines() == "Hello."

    def test_break_lines__2(self):
        text = ("- Isn't he off on Saturdays? "
                "- Didn't he tell you?")

        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "- Isn't he off on Saturdays?\n"
            "- Didn't he tell you?")

    def test_break_lines__3(self):
        text = ("Close by the king's castle "
                "lay a great dark forest.")

        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "Close by the king's castle\n"
            "lay a great dark forest.")

    def test_break_lines__4(self):
        text = ("The king's child went out "
                "into the forest and sat down "
                "by the side of the cool fountain.")

        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "The king's child went out\n"
            "into the forest and sat down\n"
            "by the side of the cool fountain.")

    def test_break_lines__5(self):
        text = ("The king's child went out "
                "into the forest and sat down by the side "
                "of the cool fountain; and when "
                "she was bored she took a golden ball, "
                "and threw it up high and caught it; "
                "and this ball was her favorite plaything.")

        self.liner.set_text(text)
        assert self.liner.break_lines() == (
            "The king's child went out\n"
            "into the forest and sat down by the side\n"
            "of the cool fountain; and when she\n"
            "was bored she took a golden ball,\n"
            "and threw it up high and caught it; and\n"
            "this ball was her favorite plaything.")



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestPositionAgent(aeidon.TestCase):

    def setup_method(self, method):
        self.project = self.new_project()

    def test_adjust_durations__gap(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end   = "00:00:02.000"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end   = "00:00:03.000"
        subtitles[2].start = "00:00:04.000"
        self.project.adjust_durations(None,
                                      gap=0.3)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end   == "00:00:01.700"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end   == "00:00:03.000"
        assert subtitles[2].start == "00:00:04.000"

    def test_adjust_durations__lengthen(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end   = "00:00:01.100"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end   = "00:00:02.100"
        subtitles[0].main_text = "1234567890"
        subtitles[1].main_text = "12345678901234567890"
        self.project.adjust_durations(None,
                                      speed=20,
                                      lengthen=True)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end   == "00:00:01.500"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end   == "00:00:03.000"

    def test_adjust_durations__maximum(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end   = "00:00:01.100"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end   = "00:00:02.100"
        self.project.adjust_durations(None,
                                      speed=10,
                                      lengthen=True,
                                      maximum=0.5)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end   == "00:00:01.500"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end   == "00:00:02.500"

    def test_adjust_durations__minimum(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end   = "00:00:01.900"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end   = "00:00:02.900"
        self.project.adjust_durations(None,
                                      speed=1000,
                                      shorten=True,
                                      minimum=0.5)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end   == "00:00:01.500"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end   == "00:00:02.500"

    def test_adjust_durations__shorten(self):
        subtitles = self.project.subtitles
        subtitles[0].start = "00:00:01.000"
        subtitles[0].end   = "00:00:01.900"
        subtitles[1].start = "00:00:02.000"
        subtitles[1].end   = "00:00:02.900"
        subtitles[0].main_text = "1234567890"
        subtitles[1].main_text = "12345678901234567890"
        self.project.adjust_durations(None,
                                      speed=100,
                                      shorten=True)

        assert subtitles[0].start == "00:00:01.000"
        assert subtitles[0].end   == "00:00:01.100"
        assert subtitles[1].start == "00:00:02.000"
        assert subtitles[1].end   == "00:00:02.200"

    def test_convert_framerate(self):
        self.project.open_main(self.new_microdvd_file(), "ascii")
        self.project.subtitles[0].start = 100
        self.project.subtitles[1].start = 200
        ifps = aeidon.framerates.FPS_23_976
        ofps = aeidon.framerates.FPS_25_000
        self.project.convert_framerate(None, ifps, ofps)
        assert self.project.framerate == ofps
        for subtitle in self.project.subtitles:
            assert subtitle.framerate == ofps
        assert self.project.subtitles[0].start == 104
        assert self.project.subtitles[1].start == 209

    def test_set_framerate(self):
        framerate = aeidon.framerates.FPS_25_000
        self.project.set_framerate(framerate)
        assert self.project.framerate == framerate
        for subtitle in self.project.subtitles:
            assert subtitle.framerate == framerate

    @aeidon.deco.reversion_test
    def test_shift_positions(self):
        orig_subtitles = [x.copy() for x in self.project.subtitles]
        indices = self.project.get_all_indices()
        self.project.shift_positions(indices, -10)
        for i, subtitle in enumerate(self.project.subtitles):
            start = orig_subtitles[i].start_frame - 10
            end = orig_subtitles[i].end_frame - 10
            assert subtitle.start_frame == start
            assert subtitle.end_frame == end

    @aeidon.deco.reversion_test
    def test_transform_positions(self):
        a, b = "00:00:01.000", "00:00:45.000"
        self.project.transform_positions(None, (2, a), (6, b))
        assert self.project.subtitles[2].start_time == a
        for subtitle in self.project.subtitles[3:6]:
            assert a < subtitle.start_time < b
        assert self.project.subtitles[6].start_time == b



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import codecs

from aeidon.i18n   import _
from unittest.mock import patch

# https://docs.python.org/3.10/library/codecs.html#standard-encodings
ENCODINGS = [
    "ascii",
    "big5",
    "big5hkscs",
    "cp037",
    "cp273",
    "cp424",
    "cp437",
    "cp500",
    "cp720",
    "cp737",
    "cp775",
    "cp850",
    "cp852",
    "cp855",
    "cp856",
    "cp857",
    "cp858",
    "cp860",
    "cp861",
    "cp862",
    "cp863",
    "cp864",
    "cp865",
    "cp866",
    "cp869",
    "cp874",
    "cp875",
    "cp932",
    "cp949",
    "cp950",
    "cp1006",
    "cp1026",
    "cp1125",
    "cp1140",
    "cp1250",
    "cp1251",
    "cp1252",
    "cp1253",
    "cp1254",
    "cp1255",
    "cp1256",
    "cp1257",
    "cp1258",
    "euc_jp",
    "euc_jis_2004",
    "euc_jisx0213",
    "euc_kr",
    "gb2312",
    "gbk",
    "gb18030",
    "hz",
    "iso2022_jp",
    "iso2022_jp_1",
    "iso2022_jp_2",
    "iso2022_jp_2004",
    "iso2022_jp_3",
    "iso2022_jp_ext",
    "iso2022_kr",
    "latin_1",
    "iso8859_2",
    "iso8859_3",
    "iso8859_4",
    "iso8859_5",
    "iso8859_6",
    "iso8859_7",
    "iso8859_8",
    "iso8859_9",
    "iso8859_10",
    "iso8859_11",
    "iso8859_13",
    "iso8859_14",
    "iso8859_15",
    "iso8859_16",
    "johab",
    "koi8_r",
    "koi8_t",
    "koi8_u",
    "kz1048",
    "mac_cyrillic",
    "mac_greek",
    "mac_iceland",
    "mac_latin2",
    "mac_roman",
    "mac_turkish",
    "ptcp154",
    "shift_jis",
    "shift_jis_2004",
    "shift_jisx0213",
    "utf_32",
    "utf_32_be",
    "utf_32_le",
    "utf_16",
    "utf_16_be",
    "utf_16_le",
    "utf_7",
    "utf_8",
    "utf_8_sig",
]


class TestModule(aeidon.TestCase):

    def test_all_found(self):
        for code in ENCODINGS:
            # Skip if code not included in the Python version
            # that we're currently running.
            if not aeidon.encodings.is_valid_code(code): continue
            code = aeidon.encodings.translate_code(code)
            assert aeidon.encodings.code_to_name(code)
            assert aeidon.encodings.code_to_description(code)

    def test_code_to_description(self):
        code_to_description = aeidon.encodings.code_to_description
        assert code_to_description("cp1006") == _("Urdu")
        assert code_to_description("hz") == _("Chinese simplified")
        assert code_to_description("shift_jis") == _("Japanese")

    def test_code_to_long_name(self):
        code, name, description = ("cp1140", "IBM1140", _("Western"))
        long_name = aeidon.encodings.code_to_long_name(code)
        assert long_name == _("{description} ({name})").format(**locals())

    def test_code_to_name(self):
        code_to_name = aeidon.encodings.code_to_name
        assert code_to_name("big5hkscs") == "Big5-HKSCS"
        assert code_to_name("cp949") == "IBM949"
        assert code_to_name("mac_roman") == "MacRoman"

    def test_detect(self):
        name = aeidon.encodings.detect(self.new_subrip_file())
        assert aeidon.encodings.is_valid_code(name)

    def test_detect_bom__none(self):
        path = self.new_subrip_file()
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding is None

    @patch("aeidon.encodings.is_valid_code", lambda x: True)
    def test_detect_bom__utf_16_be(self):
        path = self.new_subrip_file()
        blob = open(path, "rb").read()
        open(path, "wb").write(codecs.BOM_UTF16_BE + blob)
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding == "utf_16_be"

    @patch("aeidon.encodings.is_valid_code", lambda x: True)
    def test_detect_bom__utf_16_le(self):
        path = self.new_subrip_file()
        blob = open(path, "rb").read()
        open(path, "wb").write(codecs.BOM_UTF16_LE + blob)
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding == "utf_16_le"

    @patch("aeidon.encodings.is_valid_code", lambda x: True)
    def test_detect_bom__utf_32_be(self):
        path = self.new_subrip_file()
        blob = open(path, "rb").read()
        open(path, "wb").write(codecs.BOM_UTF32_BE + blob)
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding == "utf_32_be"

    @patch("aeidon.encodings.is_valid_code", lambda x: True)
    def test_detect_bom__utf_32_le(self):
        path = self.new_subrip_file()
        blob = open(path, "rb").read()
        open(path, "wb").write(codecs.BOM_UTF32_LE + blob)
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding == "utf_32_le"

    @patch("aeidon.encodings.is_valid_code", lambda x: True)
    def test_detect_bom__utf_8_sig(self):
        path = self.new_subrip_file()
        blob = open(path, "rb").read()
        open(path, "wb").write(codecs.BOM_UTF8 + blob)
        encoding = aeidon.encodings.detect_bom(path)
        assert encoding == "utf_8_sig"

    def test_get_locale_code(self):
        code = aeidon.encodings.get_locale_code()
        assert aeidon.encodings.is_valid_code(code)

    def test_get_locale_long_name(self):
        long_name = aeidon.encodings.get_locale_long_name()
        code = aeidon.encodings.get_locale_code()
        name = aeidon.encodings.code_to_name(code)
        assert long_name == _("Current locale ({})").format(name)

    def test_get_valid(self):
        assert aeidon.encodings.get_valid()
        for item in aeidon.encodings.get_valid():
            code, name, description = item
            assert aeidon.encodings.is_valid_code(code)

    def test_is_valid_code(self):
        assert aeidon.encodings.is_valid_code("gbk")
        assert aeidon.encodings.is_valid_code("utf_16_be")
        assert not aeidon.encodings.is_valid_code("xxxxx")

    def test_name_to_code(self):
        name_to_code = aeidon.encodings.name_to_code
        assert name_to_code("IBM037") == "cp037"
        assert name_to_code("GB2312") == "gb2312"
        assert name_to_code("PTCP154") == "ptcp154"

    def test_translate_code(self):
        translate_code = aeidon.encodings.translate_code
        assert translate_code("johab") == "johab"
        assert translate_code("UTF-8") == "utf_8"
        assert translate_code("ISO-8859-1") == "latin_1"



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestFinder(aeidon.TestCase):

    ####### 00000000001111111111222222 222233333333334444444444555555
    ####### 01234567890123456789012345 678901234567890123456789012345
    text = "One only risks it, because\none's survival depends on it."

    def find_indices(self, next):
        advance = self.finder.next if next else self.finder.previous
        pos = []
        while True:
            try:
                advance()
            except StopIteration:
                break
            pos.append(self.finder.pos)
        return pos

    def setup_method(self, method):
        self.finder = aeidon.Finder()
        self.finder.set_text(self.text)

    def test_next__regex(self):
        self.finder.set_regex(r"^")
        pos = self.find_indices(next=True)
        assert pos == [0, 27]

    def test_next__regex_ignore_case(self):
        self.finder.ignore_case = True
        self.finder.set_regex(r"O")
        pos = self.find_indices(next=True)
        assert pos == [1, 5, 28, 51]

    def test_next__string(self):
        self.finder.pattern = "it"
        pos = self.find_indices(next=True)
        assert pos == [17, 55]

    def test_next__string_ignore_case(self):
        self.finder.ignore_case = True
        self.finder.pattern = "o"
        pos = self.find_indices(next=True)
        assert pos == [1, 5, 28, 51]

    def test_previous__regex(self):
        self.finder.set_regex(r"\s")
        pos = self.find_indices(next=False)
        assert pos == [52, 49, 41, 32, 26, 18, 14, 8, 3]

    def test_previous__regex_ignore_case(self):
        self.finder.ignore_case = True
        self.finder.set_regex(r"O")
        pos = self.find_indices(next=False)
        assert pos == [50, 27, 4, 0]

    def test_previous__string(self):
        self.finder.pattern = "it"
        pos = self.find_indices(next=False)
        assert pos == [53, 15]

    def test_previous__string_ignore_case(self):
        self.finder.ignore_case = True
        self.finder.pattern = "o"
        pos = self.find_indices(next=False)
        assert pos == [50, 27, 4, 0]

    def test_replace__equal_length_next(self):
        self.finder.pattern = "ne"
        self.finder.replacement = "--"
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "O-- only risks it, because\n"
            "one's survival depends on it.")
        assert self.finder.pos == 3
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "O-- only risks it, because\n"
            "o--'s survival depends on it.")
        assert self.finder.pos == 30
        self.assert_raises(StopIteration, self.finder.next)

    def test_replace__equal_length_previous(self):
        self.finder.pattern = "ne"
        self.finder.replacement = "--"
        self.finder.previous()
        self.finder.replace(next=False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "o--'s survival depends on it.")
        assert self.finder.pos == 28
        self.finder.previous()
        self.finder.replace(next=False)
        assert self.finder.text == (
            "O-- only risks it, because\n"
            "o--'s survival depends on it.")
        assert self.finder.pos == 1
        self.assert_raises(StopIteration, self.finder.previous)

    def test_replace__lengthen_next(self):
        self.finder.set_regex(r"$")
        self.finder.replacement = "--"
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it, because--\n"
            "one's survival depends on it.")
        assert self.finder.pos == 28
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it, because--\n"
            "one's survival depends on it.--")
        assert self.finder.pos == 60
        self.assert_raises(StopIteration, self.finder.next)

    def test_replace__lengthen_previous(self):
        self.finder.set_regex(r"$")
        self.finder.replacement = "--"
        self.finder.previous()
        self.finder.replace(next=False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survival depends on it.--")
        assert self.finder.pos == 56
        self.finder.previous()
        self.finder.replace(next=False)
        assert self.finder.text == (
            "One only risks it, because--\n"
            "one's survival depends on it.--")
        assert self.finder.pos == 26
        self.assert_raises(StopIteration, self.finder.previous)

    def test_replace__shorten_next(self):
        self.finder.set_regex(r"[.,]")
        self.finder.replacement = ""
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it because\n"
            "one's survival depends on it.")
        assert self.finder.pos == 17
        self.finder.next()
        self.finder.replace()
        assert self.finder.text == (
            "One only risks it because\n"
            "one's survival depends on it")
        assert self.finder.pos == 54
        self.assert_raises(StopIteration, self.finder.next)

    def test_replace__shorten_previous(self):
        self.finder.set_regex(r"[.,]")
        self.finder.replacement = ""
        self.finder.previous()
        self.finder.replace(next=False)
        assert self.finder.text == (
            "One only risks it, because\n"
            "one's survival depends on it")
        assert self.finder.pos == 55
        self.finder.previous()
        self.finder.replace(next=False)
        assert self.finder.text == (
            "One only risks it because\n"
            "one's survival depends on it")
        assert self.finder.pos == 17
        self.assert_raises(StopIteration, self.finder.previous)

    def test_replace_all__regex(self):
        self.finder.set_regex(r"\s")
        self.finder.replacement = ""
        count = self.finder.replace_all()
        assert count == 9
        assert self.finder.text == (
            "Oneonlyrisksit,because"
            "one'ssurvivaldependsonit.")

    def test_replace_all__string(self):
        self.finder.pattern = "i"
        self.finder.replacement = "-"
        count = self.finder.replace_all()
        assert count == 4
        assert self.finder.text == (
            "One only r-sks -t, because\n"
            "one's surv-val depends on -t.")



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol

from gi.repository import Gtk
from unittest.mock import patch


class TestOpenAgent(gaupol.TestCase):

    def run__show_encoding_error_dialog(self):
        self.delegate._show_encoding_error_dialog("test")

    def run__show_format_error_dialog(self):
        self.delegate._show_format_error_dialog("test")

    def run__show_io_error_dialog(self):
        self.delegate._show_io_error_dialog("test", "test")

    def run__show_parse_error_dialog(self):
        self.delegate._show_parse_error_dialog("test", aeidon.formats.SUBRIP)

    @aeidon.deco.silent(gaupol.Default)
    def run__show_size_warning_dialog(self):
        self.delegate._show_size_warning_dialog("test", 2)

    @aeidon.deco.silent(gaupol.Default)
    def run__show_sort_warning_dialog(self):
        self.delegate._show_sort_warning_dialog("test", 3)

    @aeidon.deco.silent(gaupol.Default)
    def run__show_translation_warning_dialog(self):
        page = self.application.get_current_page()
        self.delegate._show_translation_warning_dialog(page)

    def setup_method(self, method):
        self.application = self.new_application()
        self.delegate = self.application.open_main.__self__

    def test_add_to_recent_files(self):
        self.application.add_to_recent_files(self.new_subrip_file(),
                                             aeidon.formats.SUBRIP,
                                             aeidon.documents.MAIN)

    def test_append_file(self):
        page = self.application.get_current_page()
        n = len(page.project.subtitles)
        self.application.append_file(self.new_subrip_file())
        assert len(page.project.subtitles) > n

    def test__on_new_project_activate(self):
        n = len(self.application.pages)
        self.application.get_action("new-project").activate()
        assert len(self.application.pages) == n + 1

    @patch("gaupol.util.flash_dialog", lambda *args: Gtk.ResponseType.OK)
    def test__on_split_project_activate(self):
        page = self.application.get_current_page()
        page.view.select_rows((3,))
        self.application.get_action("split-project").activate()

    def test_open_main(self):
        n = len(self.application.pages)
        path = self.new_subrip_file()
        self.application.open_main(path)
        assert len(self.application.pages) == n + 1

    def test_open_translation(self):
        path = self.new_subrip_file()
        self.application.open_translation(path)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol

from gi.repository import Gtk
from gaupol.dialogs.test.test_file import _TestFileDialog


class TestMultiSaveDialog(_TestFileDialog):

    def run_dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    @aeidon.deco.silent(gaupol.Default)
    def run__show_overwrite_question_dialog(self):
        files = [x.project.main_file for x in self.application.pages]
        self.dialog._show_overwrite_question_dialog(files, "test")

    def setup_method(self, method):
        self.application = self.new_application()
        self.application.add_page(self.new_page())
        self.application.add_page(self.new_page())
        modes = [x.project.main_file.mode for x in self.application.pages]
        self.dialog = gaupol.MultiSaveDialog(parent=Gtk.Window(),
                                             application=self.application,
                                             modes=modes)

        self.dialog.show()

    def test__on_response(self):
        self.dialog.response(Gtk.ResponseType.CANCEL)



# -*- coding: utf-8 -*-

# Copyright (C) 2007 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon

from unittest.mock import patch


class TestMetadataItem(aeidon.TestCase):

    def assert_name_in_locale(self, code, modifier):
        with patch("aeidon.locales.get_system_code", lambda: code):
            with patch("aeidon.locales.get_system_modifier", lambda: modifier):
                self.item.set_field("Name", "system")
                key = "{}@{}".format(code, modifier) if modifier else code
                self.item.set_field("Name[{}]".format(key), "local")
                assert self.item.get_name(localize=True) == "local"

    def setup_method(self, method):
        self.item = aeidon.MetadataItem()

    def test_get_description(self):
        self.item.set_field("Description", "test")
        assert self.item.get_description(localize=False) == "test"
        assert self.item.get_description(localize=True) == "test"

    def test_get_field(self):
        assert self.item.get_field("Test") is None
        self.item.set_field("Test", "test")
        assert self.item.get_field("Test") == "test"

    def test_get_field_boolean(self):
        assert self.item.get_field("Test") is None
        self.item.set_field("Test", "True")
        assert self.item.get_field_boolean("Test") is True

    def test_get_field_list(self):
        assert self.item.get_field("Test") is None
        self.item.set_field("Test", "Yee;Haw;")
        assert self.item.get_field_list("Test") == ["Yee", "Haw"]

    def test_get_name(self):
        self.item.set_field("Name", "test")
        assert self.item.get_name(localize=False) == "test"
        assert self.item.get_name(localize=True) == "test"

    def test_get_name__localize(self):
        self.assert_name_in_locale("en_US", "Latn")
        self.assert_name_in_locale("en_US", None)
        self.assert_name_in_locale("en", "Latn")
        self.assert_name_in_locale("en", None)

    @patch("aeidon.locales.get_system_code", lambda: None)
    def test_get_name__no_locale(self):
        self.item.set_field("Name", "system")
        assert self.item.get_name() == "system"

    def test_has_field(self):
        assert not self.item.has_field("Test")
        self.item.set_field("Test", "test")
        assert self.item.has_field("Test")

    def test_set_field(self):
        self.item.set_field("Test", "test")
        assert self.item.get_field("Test") == "test"



# -*- coding: utf-8 -*-

# Copyright (C) 2008 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol


class TestExtensionManager(gaupol.TestCase):

    def setup_method(self, method):
        gaupol.conf.path = aeidon.temp.create(".conf")
        self.manager = gaupol.ExtensionManager(self.new_application())
        gaupol.conf.extensions.active = ["custom-framerates", "none"]
        self.manager.find_extensions()
        self.manager.setup_extensions()

    def teardown_method(self, method):
        self.manager.teardown_extensions()
        gaupol.TestCase.teardown_method(self, method)

    def test_setup_extension(self):
        self.manager.setup_extension("custom-framerates")
        assert self.manager.is_active("custom-framerates")

    def test_teardown_extension(self):
        self.manager.setup_extension("custom-framerates")
        self.manager.teardown_extension("custom-framerates")
        assert not self.manager.is_active("custom-framerates")



# -*- coding: utf-8 -*-

# Copyright (C) 2013 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import gaupol

from gi.repository import Gtk


class TestFloatingLabel(gaupol.TestCase):

    def run(self):
        self.label.flash_text("Testing floating label")
        self.window.connect("notify::visible", Gtk.main_quit)
        Gtk.main()

    def setup_method(self, method):
        self.window = Gtk.Window()
        gaupol.style.load_css(self.window)
        self.window.set_default_size(800, 480)
        self.window.connect("delete-event", Gtk.main_quit)
        self.overlay = Gtk.Overlay()
        self.overlay.add(Gtk.Label())
        self.window.add(self.overlay)
        self.label = gaupol.FloatingLabel()
        self.overlay.add_overlay(self.label)
        self.window.show_all()



# -*- coding: utf-8 -*-

# Copyright (C) 2007 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol

from gi.repository import Gtk
from unittest.mock import patch

from gaupol.assistants import CapitalizationPage
from gaupol.assistants import CommonErrorPage
from gaupol.assistants import ConfirmationPage
from gaupol.assistants import HearingImpairedPage
from gaupol.assistants import IntroductionPage
from gaupol.assistants import JoinSplitWordsPage
from gaupol.assistants import LineBreakOptionsPage
from gaupol.assistants import LineBreakPage
from gaupol.assistants import ProgressPage

OK = lambda *args: Gtk.ResponseType.OK


class _TestBuilderPage(gaupol.TestCase):

    def run_page(self):
        self.window.connect("delete-event", Gtk.main_quit)
        Gtk.main()


class TestIntroductionPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = IntroductionPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class _TestLocalePage(_TestBuilderPage):

    def test_correct_texts(self):
        self.page.correct_texts(self.new_project(),
                                None,
                                aeidon.documents.MAIN)


class TestCapitalizationPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = CapitalizationPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class TestCommonErrorPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = CommonErrorPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class TestHearingImpairedPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = HearingImpairedPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class TestJoinSplitWordsPage(_TestBuilderPage):

    def run__show_error_dialog(self):
        self.page._show_error_dialog("test")

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.project = self.new_project()
        self.page = JoinSplitWordsPage(self.window)
        self.window.add(self.page)
        self.window.show_all()

    @patch("gaupol.util.flash_dialog", OK)
    def test_correct_texts(self):
        language = self.get_spell_check_language("en")
        gaupol.conf.spell_check.language = language
        self.page.correct_texts(self.project,
                                None,
                                aeidon.documents.MAIN)


class TestLineBreakPage(_TestLocalePage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = LineBreakPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class TestLineBreakOptionsPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = LineBreakOptionsPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class TestProgressPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = ProgressPage(self.window)
        self.page.reset(100)
        self.window.add(self.page)
        self.window.show_all()


class TestConfirmationPage(_TestBuilderPage):

    def setup_method(self, method):
        self.window = Gtk.Window()
        self.window.set_border_width(12)
        self.window.set_default_size(800, 500)
        self.page = ConfirmationPage(self.window)
        self.window.add(self.page)
        self.window.show_all()


class TestTextAssistant(gaupol.TestCase):

    def run_assistant(self):
        self.assistant.show()
        self.assistant.connect("apply", Gtk.main_quit)
        self.assistant.connect("cancel", Gtk.main_quit)
        self.assistant.connect("delete-event", Gtk.main_quit)
        Gtk.main()

    def setup_method(self, method):
        gaupol.conf.text_assistant.pages = ["common-error"]
        self.application = self.new_application()
        self.assistant = gaupol.TextAssistant(
            self.application.window, self.application)
        self.assistant.show()



#!/usr/bin/env python3

"""setuptools/wheel/PyPI version of the aeidon package."""

import shutil

from setup import get_aeidon_version
from setuptools import find_packages
from setuptools import setup

# Copy data files to the aeidon package, so they can be included.
shutil.copytree("data/headers", "aeidon/data/headers")
shutil.copytree("data/patterns", "aeidon/data/patterns")

setup(
    name="aeidon",
    version=get_aeidon_version(),
    author="Osmo Salomaa",
    author_email="otsaloma@iki.fi",
    description="Reading, writing and manipulating text-based subtitle files",
    long_description=open("README.aeidon.md", "r").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/otsaloma/gaupol",
    license="GPL",
    packages=find_packages(exclude=["gaupol*", "*.test"]),
    package_data={"aeidon": ["data/*/*"]},
    python_requires=">=3.5.0",
    install_requires=["charset-normalizer>2.0"],
)

shutil.rmtree("aeidon/data")



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestMarkup(aeidon.TestCase):

    text = "All things weird are normal\nin this whore of cities."

    def setup_method(self, method):
        self.markup = aeidon.Markup()

    def test_clean(self):
        text = self.markup.clean(self.text)
        assert text == self.text

    def test_decode(self):
        text = self.markup.decode(self.text)
        assert text == self.text

    def test_encode__b(self):
        text = "<b>All</b> things weird are normal\nin this whore of cities."
        assert self.markup.encode(text) == self.text

    def test_encode__color(self):
        text = "<color=#ffffff>All</color> things weird are normal\nin this whore of cities."
        assert self.markup.encode(text) == self.text

    def test_encode__font(self):
        text = "<font=sans>All things weird are normal\nin this whore of cities.</font>"
        assert self.markup.encode(text) == self.text

    def test_encode__i(self):
        text = "<i>All things weird are normal\nin this whore of cities.</i>"
        assert self.markup.encode(text) == self.text

    def test_encode__size(self):
        text = "All things weird are normal\nin this whore of <size=12>cities</size>."
        assert self.markup.encode(text) == self.text

    def test_encode__u(self):
        text = "All things weird are normal\nin this whore of <u>cities</u>."
        assert self.markup.encode(text) == self.text



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestMicroDVD(aeidon.TestCase):

    text = ("All things weird are normal\n"
            "in this whore of cities.")

    def setup_method(self, method):
        self.markup = aeidon.markups.new(aeidon.formats.MICRODVD)

    def test_bolden(self):
        assert self.markup.bolden(self.text, (0, 27)) == (
            "{y:b}All things weird are normal\n"
            "in this whore of cities.")

    def test_colorize(self):
        assert self.markup.colorize(self.text, "ccff00") == (
            "{C:$00ffcc}All things weird are normal\n"
            "in this whore of cities.")

    def test_decode__bold(self):
        text = ("{Y:b}All things weird are normal\n"
                "in this whore of cities.")
        assert self.markup.decode(text) == (
            "<b>All things weird are normal\n"
            "in this whore of cities.</b>")

    def test_decode__color(self):
        text = ("All things weird are normal\n"
                "{c:$0000ff}in this whore of cities.")
        assert self.markup.decode(text) == (
            "All things weird are normal\n"
            "<color=#ff0000>in this whore of cities.</color>")

    def test_decode__combined(self):
        text = ("{Y:bi}All things weird are normal\n"
                "{y:u}in this whore of cities.")
        assert self.markup.decode(text) == (
            "<b><i>All things weird are normal\n"
            "<u>in this whore of cities.</u></i></b>")

    def test_decode__font(self):
        text = ("{f:sans}All things weird are normal\n"
                "in this whore of cities.")
        assert self.markup.decode(text) == (
            "<font=sans>All things weird are normal</font>\n"
            "in this whore of cities.")

    def test_decode__italic(self):
        text = ("{y:i}All things weird are normal\n"
                "{y:i}in this whore of cities.")
        assert self.markup.decode(text) == (
            "<i>All things weird are normal</i>\n"
            "<i>in this whore of cities.</i>")

    def test_decode__size(self):
        text = ("All things weird are normal\n"
                "{s:12}in this whore of cities.")
        assert self.markup.decode(text) == (
            "All things weird are normal\n"
            "<size=12>in this whore of cities.</size>")

    def test_decode__underline(self):
        text = ("{Y:u}All things weird are normal\n"
                "in this whore of cities.")
        assert self.markup.decode(text) == (
            "<u>All things weird are normal\n"
            "in this whore of cities.</u>")

    def test_encode__bold(self):
        text = ("<b>All things weird are normal\n"
                "in this whore of cities.</b>")
        assert self.markup.encode(text) == (
            "{Y:b}All things weird are normal\n"
            "in this whore of cities.")

    def test_encode__color(self):
        text = ("<color=#ccccff>All things weird are normal\n"
                "in this whore of cities.</color>")
        assert self.markup.encode(text) == (
            "{C:$ffcccc}All things weird are normal\n"
            "in this whore of cities.")

    def test_encode__font(self):
        text = ("<font=sans>All things weird are normal\n"
                "in this whore of cities.</font>")
        assert self.markup.encode(text) == (
            "{F:sans}All things weird are normal\n"
            "in this whore of cities.")

    def test_encode__italic(self):
        text = ("All <i>things</i> weird are normal\n"
                "in <i>this</i> whore of cities.")
        assert self.markup.encode(text) == (
            "All things weird are normal\n"
            "in this whore of cities.")

    def test_encode__size(self):
        text = ("All things weird are normal\n"
                "<size=12>in this whore of cities</size>.")
        assert self.markup.encode(text) == (
            "All things weird are normal\n"
            "{s:12}in this whore of cities.")

    def test_encode__underline(self):
        text = ("All things weird are normal\n"
                "<u>in this whore of cities</u>.")
        assert self.markup.encode(text) == (
            "All things weird are normal\n"
            "{y:u}in this whore of cities.")

    def test_fontify(self):
        assert self.markup.fontify(self.text, "sans") == (
            "{F:sans}All things weird are normal\n"
            "in this whore of cities.")

    def test_italic_tag(self):
        assert self.markup.italic_tag.match("{Y:i}")
        assert self.markup.italic_tag.match("{y:i}")

    def test_italicize(self):
        assert self.markup.italicize(self.text) == (
            "{Y:i}All things weird are normal\n"
            "in this whore of cities.")

    def test_scale(self):
        assert self.markup.scale(self.text, 12) == (
            "{S:12}All things weird are normal\n"
            "in this whore of cities.")

    def test_tag(self):
        assert self.markup.tag.match("{y:ibu}")
        assert self.markup.tag.match("{c:$000000}")

    def test_underline(self):
        assert self.markup.underline(self.text, (0, 3)) == (
            "All things weird are normal\n"
            "in this whore of cities.")



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon


class TestMPL2(aeidon.TestCase):

    format = aeidon.formats.MPL2

    def setup_method(self, method):
        self.file = aeidon.files.new(self.format,
                                     self.new_temp_file(self.format),
                                     "ascii")

    def test_read(self):
        assert self.file.read()

    def test_write(self):
        self.file.write(self.file.read(), aeidon.documents.MAIN)
        text = open(self.file.path, "r").read().strip()
        assert text == self.get_sample_text(self.format)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol

from gi.repository import Gtk
from unittest.mock import patch

OK = lambda *args: Gtk.ResponseType.OK
CANCEL = lambda *args: Gtk.ResponseType.CANCEL


class TestToolsAgent(gaupol.TestCase):

    def setup_method(self, method):
        self.application = self.new_application()

    @patch("gaupol.util.flash_dialog", CANCEL)
    def test__on_adjust_durations_activate(self):
        self.application.get_action("adjust-durations").activate()

    @patch("gaupol.util.flash_dialog", OK)
    def test__on_check_spelling_activate(self):
        language = self.get_spell_check_language("en")
        gaupol.conf.spell_check.language = language
        self.application.get_action("check-spelling").activate()

    @patch("gaupol.util.flash_dialog", OK)
    def test__on_configure_spell_check_activate(self):
        self.application.get_action("configure-spell-check").activate()

    @patch("gaupol.util.flash_dialog", CANCEL)
    def test__on_convert_framerate_activate(self):
        self.application.get_action("convert-framerate").activate()

    @patch("gaupol.TextAssistant.show", lambda *args: None)
    def test__on_correct_texts_activate(self):
        self.application.get_action("correct-texts").activate()

    @patch("gaupol.util.flash_dialog", CANCEL)
    def test__on_shift_positions_activate__frame(self):
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.FRAME
        self.application.get_action("shift-positions").activate()

    @patch("gaupol.util.flash_dialog", CANCEL)
    def test__on_shift_positions_activate__time(self):
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.TIME
        self.application.get_action("shift-positions").activate()

    @patch("gaupol.util.flash_dialog", CANCEL)
    def test__on_transform_positions_activate__frame(self):
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.FRAME
        self.application.get_action("transform-positions").activate()

    @patch("gaupol.util.flash_dialog", CANCEL)
    def test__on_transform_positions_activate__time(self):
        page = self.application.get_current_page()
        page.edit_mode = aeidon.modes.TIME
        self.application.get_action("transform-positions").activate()



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import codecs


class PuppetSubtitleFile(aeidon.SubtitleFile):

    format = aeidon.formats.SUBVIEWER2
    mode = aeidon.modes.TIME


class TestSubtitleFile(aeidon.TestCase):

    def setup_method(self, method):
        path = self.new_temp_file(PuppetSubtitleFile.format)
        newline = aeidon.newlines.UNIX
        self.file = PuppetSubtitleFile(path, "ascii", newline)

    def test_read__utf_16(self):
        path = self.new_subrip_file()
        with open(path, "r") as f:
            text = f.read()
        with open(path, "w", encoding="utf_16") as f:
            f.write(text)
        file = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_16")
        file.read()

    def test_read__utf_16_be(self):
        path = self.new_subrip_file()
        with open(path, "r") as f:
            text = f.read()
        with open(path, "w", encoding="utf_16_be") as f:
            f.write(str(codecs.BOM_UTF16_BE, "utf_16_be"))
            f.write(text)
        file = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_16_be")
        file.read()

    def test_read__utf_16_le(self):
        path = self.new_subrip_file()
        with open(path, "r") as f:
            text = f.read()
        with open(path, "w", encoding="utf_16_le") as f:
            f.write(str(codecs.BOM_UTF16_LE, "utf_16_le"))
            f.write(text)
        file = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_16_le")
        file.read()

    def test_read__utf_8_sig(self):
        path = self.new_subrip_file()
        with open(path, "r") as f:
            text = f.read()
        with open(path, "w", encoding="utf_8_sig") as f:
            f.write(text)
        file = aeidon.files.new(aeidon.formats.SUBRIP, path, "utf_8")
        file.read()



#!/usr/bin/env python3

"""
There are four relevant customizations to the standard distutils installation
process: (1) allowing separate installations of aeidon and gaupol, (2) writing
the aeidon.paths module, (3) handling translations and (4) extensions.

(1) Allowing separate installations of aeidon and gaupol are handled through
    global options --with-aeidon, --without-aeidon, --with-gaupol and
    --without-gaupol. See 'python3 setup.py --help' and the file
    'README.aeidon.md' for documentation and the Distribution class defined
    in this file for the implementation and logic of how that is handled.

(2) Gaupol finds non-Python files based on the paths written in module
    aeidon.paths. In aeidon/paths.py the paths default to the ones in the
    source directory. During the 'install_lib' command the file gets rewritten
    to build/aeidon/paths.py with the installation paths and that file will be
    installed. The paths are based on variable 'install_data' with the 'root'
    variable stripped if it is given. If doing distro-packaging, make sure this
    file gets correctly written.

(3) During installation, the .po files are compiled into .mo files and the
    appdata, desktop, pattern and extension metadata files are translated.
    This requires gettext.

(4) Extensions are installed under the data directory. All python code included
    in the extensions are compiled during the 'install_data' command, using the
    same arguments for 'byte_compile' as used by the 'install_lib' command.
    If the 'install_lib' command was given the '--no-compile' option, then
    extensions are not compiled either.
"""

import glob
import logging
import os
import re
import shutil
import sys

logging.basicConfig(level=logging.INFO, format="%(levelname)s: %(message)s")
log = logging.getLogger(__name__)
log.setLevel(logging.INFO)

try:
    import distutils
except ImportError:
    # Use distutils from setuptools, needed since Python 3.12.
    # https://docs.python.org/3/whatsnew/3.12.html#distutils
    import setuptools._distutils as distutils

from distutils.command.clean import clean
from distutils.command.install import install
from distutils.command.install_data import install_data
from distutils.command.install_lib import install_lib
from distutils.dist import Distribution as distribution


distribution.global_options.extend([
    ("mandir=", None, "relative installation directory for man pages (defaults to 'share/man')"),
    ("with-aeidon", None, "install the aeidon package"),
    ("without-aeidon", None, "don't install the aeidon package"),
    ("with-gaupol", None, "install the gaupol package"),
    ("without-gaupol", None, "don't install the gaupol package"),
    ("with-iso-codes", None, "install iso-codes JSON files"),
    ("without-iso-codes", None, "don't install iso-codes JSON files"),
])

distribution.negative_opt.update({
    "without-aeidon": "with-aeidon",
    "without-gaupol": "with-gaupol",
    "without-iso-codes": "with-iso-codes",
})


def get_aeidon_version():
    path = os.path.join("aeidon", "__init__.py")
    text = open(path, "r", encoding="utf_8").read()
    return re.search(r"__version__ *= *['\"](.*?)['\"]", text).group(1)

def get_gaupol_version():
    path = os.path.join("gaupol", "__init__.py")
    text = open(path, "r", encoding="utf_8").read()
    return re.search(r"__version__ *= *['\"](.*?)['\"]", text).group(1)

def run_or_exit(cmd):
    if os.system(cmd) == 0: return
    log.error("command {!r} failed".format(cmd))
    raise SystemExit(1)

def run_or_warn(cmd):
    if os.system(cmd) == 0: return
    log.warning("command {!r} failed".format(cmd))


class Clean(clean):

    def run(self):
        clean.run(self)
        f = open(os.path.join("manifests", "clean.manifest"), "r")
        for targets in [glob.glob(x.strip()) for x in f]:
            for target in filter(os.path.isdir, targets):
                log.info("removing {}".format(target))
                if not self.dry_run:
                    shutil.rmtree(target)
            for target in filter(os.path.isfile, targets):
                log.info("removing {}".format(target))
                if not self.dry_run:
                    os.remove(target)
        f.close()


class Distribution(distribution):

    def __init__(self, attrs=None):
        if sys.platform == "win32":
            self.executables = []
        self.mandir = "share/man"
        self.with_aeidon = True
        self.with_gaupol = True
        self.with_iso_codes = True
        distribution.__init__(self, attrs)
        self.data_files = []
        self.packages = []
        self.scripts = []

    def __find_data_files(self, name):
        fok = lambda x: not x.endswith((".in", ".pyc"))
        basename = "{}.manifest".format(name)
        f = open(os.path.join("manifests", basename), "r")
        for line in [x.strip() for x in f]:
            if not line: continue
            if line.startswith("["):
                dest = line[1:-1]
                continue
            files = list(filter(fok, glob.glob(line)))
            files = list(filter(os.path.isfile, files))
            assert files
            self.data_files.append((dest, files))
        f.close()

    def __find_man_pages(self):
        mandir = self.mandir
        while mandir.endswith("/"):
            mandir = mandir[:-1]
        dest = "{}/man1".format(mandir)
        self.data_files.append((dest, ["data/gaupol.1"]))

    def __find_packages(self, name):
        for root, dirs, files in os.walk(name):
            init_path = os.path.join(root, "__init__.py")
            if not os.path.isfile(init_path): continue
            path = root.replace(os.sep, ".")
            if path.endswith(".test"): continue
            self.packages.append(path[path.find(name):])

    def __find_scripts(self, name):
        if name == "gaupol":
            self.scripts.append("bin/gaupol")

    def parse_command_line(self):
        value = distribution.parse_command_line(self)
        # Configuration files are parsed before the command line, so once the
        # command line is parsed, all options have their final values and thus
        # files corresponding to the packages to install can finally be set.
        if self.with_aeidon:
            self.__find_data_files("aeidon")
            self.__find_packages("aeidon")
        if self.with_aeidon and self.with_iso_codes:
            self.__find_data_files("iso-codes")
        if self.with_gaupol:
            self.__find_data_files("gaupol")
            self.__find_man_pages()
            self.__find_packages("gaupol")
            self.__find_scripts("gaupol")
        # Redefine name, version and requires metadata attributes. These are
        # used in the egg-info file written by distutils. While egg-info files
        # appear completely useless, it needs to be ensured that if aeidon and
        # gaupol are installed separately, they install differently named
        # egg-info files in order to avoid overwriting each other.
        if self.with_gaupol:
            self.metadata.name = "gaupol"
            self.metadata.version = get_gaupol_version()
        else: # without gaupol
            self.metadata.name = "aeidon"
            self.metadata.version = get_aeidon_version()
        return value

    def parse_config_files(self, filenames=None):
        value = distribution.parse_config_files(self, filenames)
        strtobool = distutils.util.strtobool
        for option in ["with_aeidon", "with_gaupol", "with_iso_codes"]:
            value = getattr(self, option)
            if isinstance(value, str):
                setattr(self, option, strtobool(value))
        return value


class Install(install):

    def run(self):
        install.run(self)
        if not self.distribution.with_gaupol: return
        get_command_obj = self.distribution.get_command_obj
        root = get_command_obj("install").root
        data_dir = get_command_obj("install_data").install_dir
        # Assume we're actually installing if --root was not given.
        if (root is not None) or (data_dir is None): return
        directory = os.path.join(data_dir, "share", "applications")
        log.info("updating desktop database in {}".format(directory))
        run_or_warn('update-desktop-database "{}"'.format(directory))


class InstallData(install_data):

    def __build_extensions(self):
        get_command_obj = self.distribution.get_command_obj
        if not get_command_obj("install_lib").compile: return
        optimize = get_command_obj("install_lib").optimize
        data_dir = get_command_obj("install_data").install_dir
        data_dir = os.path.join(data_dir, "share", "gaupol")
        files = glob.glob("{}/extensions/*/*.py".format(data_dir))
        distutils.util.byte_compile(files, optimize, self.force, self.dry_run)
        # Figure out paths of the compiled files and add them to
        # self.outfiles so that 'setup.py --record' works correctly.
        def get_cache_pattern(path):
            dir, file = os.path.split(path)
            file = os.path.splitext(file)[0] + ".*"
            return os.path.join(dir, "__pycache__", file)
        for pattern in map(get_cache_pattern, files):
            for file in glob.glob(pattern):
                if not file in self.outfiles:
                    self.outfiles.append(file)

    def __generate_linguas(self):
        linguas = sorted(glob.glob("po/*.po"))
        linguas = [os.path.basename(x)[:-3] for x in linguas]
        with open("po/LINGUAS", "w") as f:
            f.write("\n".join(linguas) + "\n")

    def __get_appdata_file(self):
        path = os.path.join("data", "io.otsaloma.gaupol.appdata.xml")
        command = "msgfmt --xml -d po --template {}.in -o {}"
        run_or_warn(command.format(path, path))
        if not os.path.isfile(path):
            # The above can fail with an old version of gettext,
            # fall back on copying the file without translations.
            shutil.copy("{}.in".format(path), path)
        return ("share/metainfo", [path])

    def __get_desktop_file(self):
        path = os.path.join("data", "io.otsaloma.gaupol.desktop")
        command = "msgfmt --desktop -d po --template {}.in -o {}"
        run_or_warn(command.format(path, path))
        if not os.path.isfile(path):
            # The above can fail with an old version of gettext,
            # fall back on copying the file without translations.
            shutil.copy("{}.in".format(path), path)
        return ("share/applications", [path])

    def __get_extension_file(self, extension_file):
        assert extension_file.endswith(".in")
        path = extension_file[:-3]
        command = " ".join((
            "msgfmt --desktop -d po --template {}.in -o {}",
            "--keyword=",
            "--keyword=Name",
            "--keyword=Description",
        ))
        run_or_warn(command.format(path, path))
        if not os.path.isfile(path):
            # The above can fail with an old version of gettext,
            # fall back on copying the file without translations.
            shutil.copy("{}.in".format(path), path)
        extension = os.path.basename(os.path.dirname(extension_file))
        return ("share/gaupol/extensions/{}".format(extension), [path])

    def __get_extension_files(self):
        files = sorted(glob.glob("data/extensions/*/*.extension.in"))
        return [self.__get_extension_file(x) for x in files]

    def __get_mo_file(self, po_file):
        locale = os.path.basename(po_file[:-3])
        mo_dir = os.path.join("locale", locale, "LC_MESSAGES")
        mo_file = os.path.join(mo_dir, "gaupol.mo")
        log.info("compiling {}".format(mo_file))
        os.makedirs(mo_dir, exist_ok=True)
        run_or_exit("msgfmt {} -o {}".format(po_file, mo_file))
        dest_dir = os.path.join("share", mo_dir)
        return (dest_dir, [mo_file])

    def __get_mo_files(self):
        if sys.platform == "win32": return []
        files = sorted(glob.glob("po/*.po"))
        return [self.__get_mo_file(x) for x in files]

    def __get_pattern_file(self, pattern_file):
        assert pattern_file.endswith(".in")
        path = pattern_file[:-3]
        command = " ".join((
            "msgfmt --desktop -d po --template {}.in -o {}",
            "--keyword=",
            "--keyword=Name",
            "--keyword=Description",
        ))
        run_or_warn(command.format(path, path))
        if not os.path.isfile(path):
            # The above can fail with an old version of gettext,
            # fall back on copying the file without translations.
            shutil.copy("{}.in".format(path), path)
        return ("share/gaupol/patterns", [path])

    def __get_pattern_files(self):
        files = sorted(glob.glob("data/patterns/*.in"))
        return [self.__get_pattern_file(x) for x in files]

    def run(self):
        self.__generate_linguas()
        if self.distribution.with_aeidon:
            self.data_files.extend(self.__get_mo_files())
            self.data_files.extend(self.__get_pattern_files())
        if self.distribution.with_gaupol:
            self.data_files.extend(self.__get_extension_files())
            self.data_files.append(self.__get_appdata_file())
            self.data_files.append(self.__get_desktop_file())
        install_data.run(self)
        if self.distribution.with_gaupol:
            self.__build_extensions()


class InstallLib(install_lib):

    def install(self):
        if not self.distribution.with_aeidon:
            return install_lib.install(self)
        get_command_obj = self.distribution.get_command_obj
        root = get_command_obj("install").root
        prefix = get_command_obj("install").install_data
        # Allow --root to be used like DESTDIR.
        if root is not None:
            prefix = os.path.abspath(prefix)
            prefix = prefix.replace(os.path.abspath(root), "")
        data_dir = os.path.join(prefix, "share", "gaupol")
        locale_dir = os.path.join(prefix, "share", "locale")
        # Write changes to the aeidon.paths module.
        path = os.path.join(self.build_dir, "aeidon", "paths.py")
        text = open(path, "r", encoding="utf_8").read()
        patt = r"^DATA_DIR = .*$"
        repl = "DATA_DIR = {!r}".format(data_dir)
        text = re.sub(patt, repl, text, flags=re.MULTILINE)
        assert text.count(repl) == 1
        patt = r"^LOCALE_DIR = .*$"
        repl = "LOCALE_DIR = {!r}".format(locale_dir)
        text = re.sub(patt, repl, text, flags=re.MULTILINE)
        assert text.count(repl) == 1
        open(path, "w", encoding="utf_8").write(text)
        return install_lib.install(self)


setup_kwargs = {
    "name": "gaupol",
    "version": get_gaupol_version(),
    "author": "Osmo Salomaa",
    "author_email": "otsaloma@iki.fi",
    "url": "https://otsaloma.io/gaupol/",
    "description": "Editor for text-based subtitle files",
    "license": "GPL",
    "distclass": Distribution,
    "cmdclass": {
        "clean": Clean,
        "install": Install,
        "install_data": InstallData,
        "install_lib": InstallLib,
    },
}

if __name__ == "__main__":
    os.chdir(os.path.dirname(__file__) or ".")
    distutils.core.setup(**setup_kwargs)



# -*- coding: utf-8 -*-

# Copyright (C) 2006 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import gaupol


class TestAttrDict(gaupol.TestCase):

    def setup_method(self, method):
        self.root = dict(test=1, rest=0)
        self.attr_dict = gaupol.AttributeDictionary(self.root)

    def test___getattr__(self):
        assert self.attr_dict.test == 1
        assert self.attr_dict.rest == 0

    def test___setattr__(self):
        self.attr_dict.test = 5
        assert self.attr_dict.test == 5
        assert self.root["test"] == 5

    def test_add_attribute(self):
        self.attr_dict.add_attribute("best", 2)
        assert self.attr_dict.best == 2
        assert self.root["best"] == 2

    def test_add_attribute__nested(self):
        root = dict(test=1, rest=0)
        self.attr_dict.add_attribute("nest", root)
        assert self.attr_dict.nest.test == 1
        assert self.attr_dict.nest.rest == 0
        assert self.root["nest"]["test"] == 1
        assert self.root["nest"]["rest"] == 0

    def test_extend(self):
        root = dict(test=3, pest=4)
        self.attr_dict.extend(root)
        assert self.attr_dict.test == 1
        assert self.attr_dict.rest == 0
        assert self.attr_dict.pest == 4
        assert self.root["test"] == 1
        assert self.root["rest"] == 0
        assert self.root["pest"] == 4

    def test_extend__nested(self):
        root = dict(test=1, rest=0)
        self.attr_dict.add_attribute("nest", root)
        root = dict(nest=dict(test=3, pest=4))
        self.attr_dict.extend(root)
        assert self.attr_dict.nest.test == 1
        assert self.attr_dict.nest.rest == 0
        assert self.attr_dict.nest.pest == 4
        assert self.root["nest"]["test"] == 1
        assert self.root["nest"]["rest"] == 0
        assert self.root["nest"]["pest"] == 4

    def test_update(self):
        root = dict(test=3, pest=4)
        self.attr_dict.update(root)
        assert self.attr_dict.test == 3
        assert self.attr_dict.rest == 0
        assert self.attr_dict.pest == 4
        assert self.root["test"] == 3
        assert self.root["rest"] == 0
        assert self.root["pest"] == 4

    def test_update__nested(self):
        root = dict(test=1, rest=0)
        self.attr_dict.add_attribute("nest", root)
        root = dict(nest=dict(test=3, pest=4))
        self.attr_dict.update(root)
        assert self.attr_dict.nest.test == 3
        assert self.attr_dict.nest.rest == 0
        assert self.attr_dict.nest.pest == 4
        assert self.root["nest"]["test"] == 3
        assert self.root["nest"]["rest"] == 0
        assert self.root["nest"]["pest"] == 4



# -*- coding: utf-8 -*-

# Copyright (C) 2011 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import sys

sys.path.insert(0, os.path.abspath(os.path.join(
    os.path.dirname(__file__), "..")))

sys.path.insert(1, os.path.abspath(os.path.join(
    os.path.dirname(__file__), "..", "..", "..", "..")))

import aeidon
import gaupol

from gi.repository import Gtk
from unittest.mock import patch


class TestAddFramerateDialog(gaupol.TestCase):

    def run_dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        module = __import__("custom-framerates", {}, {}, [])
        self.dialog = module.AddFramerateDialog(Gtk.Window())
        self.dialog.show()

    def test_get_framerate(self):
        assert self.dialog.get_framerate() == 0.0


class TestPreferencesDialog(gaupol.TestCase):

    def run_dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        module = __import__("custom-framerates", {}, {}, [])
        self.framerates = [20.0, 21.0, 22.0, 23.0]
        self.dialog = module.PreferencesDialog(self.framerates, Gtk.Window())
        self.dialog.show()

    @patch("gaupol.util.run_dialog", lambda *args: Gtk.ResponseType.OK)
    def test__on_add_button_clicked(self):
        orig_framerates = self.dialog.get_framerates()
        self.dialog._add_button.emit("clicked")
        framerates = self.dialog.get_framerates()
        assert len(framerates) == len(orig_framerates) + 1
        assert 0.0 in framerates

    def test__on_remove_button_clicked(self):
        orig_framerates = self.dialog.get_framerates()
        selection = self.dialog._tree_view.get_selection()
        selection.unselect_all()
        path = gaupol.util.tree_row_to_path(0)
        selection.select_path(path)
        self.dialog._remove_button.emit("clicked")
        framerates = self.dialog.get_framerates()
        assert len(framerates) == len(orig_framerates) - 1

    def test_get_framerates(self):
        framerates = self.dialog.get_framerates()
        assert framerates == self.framerates


class TestCustomFrameratesExtension(gaupol.TestCase):

    def setup_method(self, method):
        module = __import__("custom-framerates", {}, {}, [])
        self.extension = module.CustomFrameratesExtension()
        self.application = self.new_application()
        self.extension.setup(self.application)
        assert hasattr(aeidon.framerates, "FPS_48_000")

    def teardown_method(self, method):
        self.extension.teardown(self.application)
        assert not hasattr(aeidon.framerates, "FPS_48_000")
        gaupol.TestCase.teardown_method(self, self.application)

    @patch("gaupol.util.run_dialog", lambda *args: Gtk.ResponseType.CLOSE)
    def test_show_preferences_dialog(self):
        self.extension.show_preferences_dialog(self.application.window)

    def test_teardown__custom(self):
        page = self.application.get_current_page()
        page.project.set_framerate(aeidon.framerates.FPS_48_000)
        self.extension.teardown(self.application)
        self.extension.setup(self.application)



# -*- coding: utf-8 -*-

# Copyright (C) 2007 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import os


class TestModule(aeidon.TestCase):

    def test_create(self):
        path = aeidon.temp.create()
        aeidon.temp.remove(path)

    def test_create_directory(self):
        path = aeidon.temp.create_directory()
        aeidon.temp.remove(path)

    def test_remove__directory(self):
        path = aeidon.temp.create_directory()
        open(os.path.join(path, "a"), "w").write("a")
        open(os.path.join(path, "b"), "w").write("b")
        os.makedirs(os.path.join(path, "c"))
        aeidon.temp.remove(path)
        assert not os.path.isdir(path)
        aeidon.temp.remove(path)

    def test_remove__file(self):
        path = aeidon.temp.create()
        aeidon.temp.remove(path)
        assert not os.path.isfile(path)
        aeidon.temp.remove(path)

    def test_remove_all(self):
        path_1 = aeidon.temp.create()
        path_2 = aeidon.temp.create()
        aeidon.temp.remove_all()
        assert not os.path.isfile(path_1)
        assert not os.path.isfile(path_2)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import gaupol

from gi.repository import Gtk


class TestSpellCheckDialog(gaupol.TestCase):

    def run_dialog(self):
        self.dialog.run()
        self.dialog.destroy()

    def setup_method(self, method):
        # Don't interact with the config files at all.
        aeidon.SpellChecker.add_to_personal = aeidon.SpellChecker.add_to_session
        aeidon.SpellChecker.read_replacements = lambda *args: None
        aeidon.SpellChecker.write_replacements = lambda *args: None
        language = self.get_spell_check_language("en")
        gaupol.conf.spell_check.language = language
        self.application = self.new_application()
        for page in self.application.pages:
            for subtitle in page.project.subtitles:
                subtitle.main_text = subtitle.main_text.replace("a", "x")
                subtitle.tran_text = subtitle.tran_text.replace("a", "x")
            page.reload_view_all()
        self.dialog = gaupol.SpellCheckDialog(self.application.window, self.application)
        self.dialog.show()

    def test__on_add_button_clicked(self):
        self.dialog._add_button.emit("clicked")

    def test__on_ignore_all_button_clicked(self):
        self.dialog._ignore_all_button.emit("clicked")

    def test__on_ignore_button_clicked(self):
        self.dialog._ignore_button.emit("clicked")

    def test__on_replace_all_button_clicked(self):
        self.dialog._replace_all_button.emit("clicked")

    def test__on_replace_button_clicked(self):
        self.dialog._replace_button.emit("clicked")

    def test__on_response(self):
        self.dialog._replace_button.emit("clicked")
        self.dialog.response(Gtk.ResponseType.CLOSE)



# -*- coding: utf-8 -*-

# Copyright (C) 2005 Osmo Salomaa
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import aeidon
import importlib

from aeidon.i18n   import _, d_
from unittest.mock import patch


class TestModule(aeidon.TestCase):

    def test_code_to_country(self):
        country = d_("iso_3166", "South Africa")
        assert aeidon.locales.code_to_country("af_ZA") == country
        assert aeidon.locales.code_to_country("af") is None

    def test_code_to_language(self):
        language = d_("iso_639", "Icelandic")
        assert aeidon.locales.code_to_language("is_IS") == language
        assert aeidon.locales.code_to_language("is") == language

    def test_code_to_name(self):
        language = d_("iso_639", "Mongolian")
        country = d_("iso_3166", "Mongolia")
        name = _("{language} ({country})").format(**locals())
        assert aeidon.locales.code_to_name("mn_MN") == name
        assert aeidon.locales.code_to_name("mn") == language

    def test_get_system_code(self):
        assert aeidon.locales.get_system_code()

    @patch.dict("os.environ", dict(LANGUAGE="sr@Latn"))
    def test_get_system_modifier__latn(self):
        importlib.reload(aeidon.locales)
        assert aeidon.locales.get_system_modifier() == "Latn"

    @patch.dict("os.environ", dict(LANGUAGE="en"))
    def test_get_system_modifier__none(self):
        importlib.reload(aeidon.locales)
        assert aeidon.locales.get_system_modifier() is None
